@@ -155,7 +155,7 @@ class TelemetryClient:
         except Exception:
             logger.warning("Failed to add log to telemetry.", exc_info=True)
 
-    def send_batch(self) -> None:
+    def send_batch(self, retry: bool = False) -> None:
         if self.is_closed:
             raise Exception("Attempted to send batch when TelemetryClient is closed")
         elif not self._enabled:
@@ -186,6 +186,7 @@ class TelemetryClient:
                 method="post",
                 client=None,
                 timeout=5,
+                _no_retry=not retry,
             )
             if not ret["success"]:
                 logger.info(
@@ -204,11 +205,10 @@ class TelemetryClient:
     def is_closed(self) -> bool:
         return self._rest is None
 
-    def close(self, send_on_close: bool = True) -> None:
+    def close(self, retry: bool = False) -> None:
         if not self.is_closed:
             logger.debug("Closing telemetry client.")
-            if send_on_close:
-                self.send_batch()
+            self.send_batch(retry=retry)
             self._rest = None
 
     def disable(self) -> None:
